@@ -19,7 +19,7 @@
     The official Delicious.com API and the JSON/RSS feeds do not provide all
     the functionality mentioned above, and in such cases this module will query
     the Delicious.com *website* directly and extract the required information
-    by parsing the HTML code of the resulting web pages (a kind of poor man's
+    by parsing the HTML code of the resulting Web pages (a kind of poor man's
     web mining). The module is able to detect IP throttling, which is employed
     by Delicious.com to temporarily block abusive HTTP request behavior, and
     will raise a custom Python error to indicate that. Please be a nice netizen
@@ -38,24 +38,24 @@
     Changelog is available at
     http://code.michael-noll.com/?p=deliciousapi;a=log
 
-    Copyright 2006-2009 Michael G. Noll <http://www.michael-noll.com/>
+    Copyright 2006-2010 Michael G. Noll <http://www.michael-noll.com/>
 
 """
 
 __author__ = "Michael G. Noll"
-__copyright__ = "(c) 2006-2009 Michael G. Noll"
-__description__ = "Unofficial Python API for retrieving data from del.icio.us / Delicious.com"
+__copyright__ = "(c) 2006-2010 Michael G. Noll"
+__description__ = "Unofficial Python API for retrieving data from Delicious.com"
 __email__ = "coding[AT]michael-REMOVEME-noll[DOT]com"
 __license__ = "GPLv2"
 __maintainer__ = "Michael G. Noll"
 __status__ = "Development"
 __url__ = "http://www.michael-noll.com/"
-__version__ = "1.5.14"
+__version__ = "1.6.1"
 
 import base64
 import cgi
 import datetime
-import md5
+import hashlib
 from operator import itemgetter
 import re
 import socket
@@ -181,6 +181,19 @@ class DeliciousURL(object):
 
         total_bookmarks:
             The number of total bookmarks (posts) of the document.
+            Note that the value of total_bookmarks can be greater than the
+            length of "bookmarks" depending on how much (detailed) bookmark
+            data could be retrieved from Delicious.com.
+
+            Here's some more background information:
+            The value of total_bookmarks is the "real" number of bookmarks of
+            URL "url" stored at Delicious.com as reported by Delicious.com
+            itself (so it's the "ground truth"). On the other hand, the length
+            of "bookmarks" depends on iteratively scraped bookmarking data.
+            Since scraping Delicous.com's Web pages has its limits in practice,
+            this means that DeliciousAPI could most likely not retrieve all
+            available bookmarks. In such a case, the value reported by
+            total_bookmarks is greater than the length of "bookmarks".
 
     """
 
@@ -215,6 +228,8 @@ class DeliciousURL(object):
         to put it is that 23 users used the tag 'foo' when
         bookmarking the URL.
 
+        @return: Dictionary mapping tags to their tag count.
+
         """
         total_tags = {}
         for user, tags, comment, timestamp in self.bookmarks:
@@ -224,7 +239,8 @@ class DeliciousURL(object):
     tags = property(fget=get_tags, doc="Returns a dictionary mapping tags to their tag count")
 
     def get_hash(self):
-        m = md5.new(self.url)
+        m = hashlib.md5()
+        m.update(self.url)
         return m.hexdigest()
     hash = property(fget=get_hash, doc="Returns the MD5 hash of the URL of this document")
 
@@ -236,7 +252,7 @@ class DeliciousAPI(object):
     Instead of using just the functionality provided by the official
     Delicious.com API (which has limited features), this class retrieves
     information from the Delicious.com website directly and extracts data from
-    the web pages.
+    the Web pages.
 
     Note that Delicious.com will block clients with too many queries in a
     certain time frame (similar to their API throttling). So be a nice citizen
@@ -253,27 +269,31 @@ class DeliciousAPI(object):
         ):
         """Set up the API module.
 
-        Parameters:
-            http_proxy (optional)
-                Use an HTTP proxy for HTTP connections. Proxy support for
-                HTTPS is not available yet.
-                Format: "hostname:port" (e.g., "localhost:8080")
+        @param http_proxy: Optional, default: "".
+            Use an HTTP proxy for HTTP connections. Proxy support for
+            HTTPS is not available yet.
+            Format: "hostname:port" (e.g., "localhost:8080")
+        @type http_proxy: str
 
-            tries (optional, default: 3):
-                Try the specified number of times when downloading a
-                monitored document fails. tries must be >= 1.
-                See also wait_seconds.
+        @param tries: Optional, default: 3.
+            Try the specified number of times when downloading a monitored
+            document fails. tries must be >= 1. See also wait_seconds.
+        @type tries: int
 
-            wait_seconds (optional, default: 3):
-                Wait the specified number of seconds before re-trying to
-                download a monitored document. wait_seconds must be >= 0.
-                See also tries.
+        @param wait_seconds: Optional, default: 3.
+            Wait the specified number of seconds before re-trying to
+            download a monitored document. wait_seconds must be >= 0.
+            See also tries.
+        @type wait_seconds: int
 
-            user_agent (optional, default: "DeliciousAPI/<version> (+http://www.michael-noll.com/wiki/Del.icio.us_Python_API)")
-                Set the User-Agent HTTP Header.
+        @param user_agent: Optional, default: "DeliciousAPI/<version>
+            (+http://www.michael-noll.com/wiki/Del.icio.us_Python_API)".
+            The User-Agent HTTP Header to use when querying Delicous.com.
+        @type user_agent: str
 
-            timeout (optional, default: 30):
-                Set network timeout. timeout must be >= 0.
+        @param timeout: Optional, default: 30.
+            Set network timeout. timeout must be >= 0.
+        @type timeout: int
 
         """
         assert tries >= 1
@@ -288,10 +308,25 @@ class DeliciousAPI(object):
 
 
     def _query(self, path, host="delicious.com", user=None, password=None, use_ssl=False):
-        """Queries delicious for information, specified by (query) path.
+        """Queries Delicious.com for information, specified by (query) path.
 
-        Returns None on errors (i.e. on all HTTP status other than 200).
-        On success, returns the content of the HTML response.
+        @param path: The HTTP query path.
+        @type path: str
+
+        @param host: The host to query, default: "delicious.com".
+        @type host: str
+
+        @param user: The Delicious.com username if any, default: None.
+        @type user: str
+
+        @param password: The Delicious.com password of user, default: None.
+        @type password: unicode/str
+
+        @param use_ssl: Whether to use SSL encryption or not, default: False.
+        @type use_ssl: bool
+
+        @return: None on errors (i.e. on all HTTP status other than 200).
+            On success, returns the content of the HTML response.
 
         """
         opener = None
@@ -361,7 +396,8 @@ class DeliciousAPI(object):
 
 
     def get_url(self, url, max_bookmarks=50, sleep_seconds=1):
-        """Returns a DeliciousURL instance representing the Delicious.com history of url.
+        """
+        Returns a DeliciousURL instance representing the Delicious.com history of url.
 
         Generally, this method is what you want for getting title, bookmark, tag,
         and user information about a URL.
@@ -372,28 +408,34 @@ class DeliciousAPI(object):
         increases linearly with the number of 50-bookmarks-chunks; i.e.
         it will take 10 times longer to retrieve 500 bookmarks than 50.
 
-        Parameters:
-            url
-                The URL of the web document to be queried for.
+        @param url: The URL of the web document to be queried for.
+        @type url: str
 
-            max_bookmarks (optional, default: 50)
-                See the documentation of get_bookmarks() for more
-                information as get_url() uses get_bookmarks() to
-                retrieve a url's bookmarking history.
+        @param max_bookmarks: Optional, default: 50.
+            See the documentation of get_bookmarks() for more information
+            as get_url() uses get_bookmarks() to retrieve a url's
+            bookmarking history.
+        @type max_bookmarks: int
 
-            sleep_seconds (optional, default: 1)
-                See the documentation of get_bookmarks() for more
-                information as get_url() uses get_bookmarks() to
-                retrieve a url's bookmarking history.
+        @param sleep_seconds: Optional, default: 1.
+            See the documentation of get_bookmarks() for more information
+            as get_url() uses get_bookmarks() to retrieve a url's
+            bookmarking history. sleep_seconds must be >= 1 to comply with
+            Delicious.com's Terms of Use.
+        @type sleep_seconds: int
+
+        @return: DeliciousURL instance representing the Delicious.com history
+            of url.
 
         """
         # we must wait at least 1 second between subsequent queries to
-        # comply with delicious' Terms of Use
+        # comply with Delicious.com's Terms of Use
         assert sleep_seconds >= 1
 
         document = DeliciousURL(url)
 
-        m = md5.new(url)
+        m = hashlib.md5()
+        m.update(url)
         hash = m.hexdigest()
 
         path = "/v2/json/urlinfo/%s" % hash
@@ -459,7 +501,7 @@ class DeliciousAPI(object):
 
         @param username: Delicous.com username for which network information is
             retrieved.
-        @type: unicode/str
+        @type username: unicode/str
 
         @return: Tuple of two lists ([<followees>, [<followers>]), where each list
             contains tuples of (username, tracking_since_timestamp).
@@ -554,58 +596,62 @@ class DeliciousAPI(object):
         """
         Returns the bookmarks of url or user, respectively.
 
-        For urls, it returns a list of (user, tags, comment, timestamp) tuples.
-        For users, it returns a list of (url, tags, title, comment, timestamp) tuples.
-
-        Bookmarks are sorted "descendingly" by creation time, i.e. newer
-        bookmarks are first.
-
-        Delicious only returns up to 50 bookmarks per URL on its website.
+        Delicious.com only returns up to 50 bookmarks per URL on its website.
         This means that we have to do subsequent queries plus parsing if
         we want to retrieve more than 50. Roughly speaking, the processing
         time of get_bookmarks() increases linearly with the number of
         50-bookmarks-chunks; i.e. it will take 10 times longer to retrieve
         500 bookmarks than 50.
 
-        Parameters:
-            url
-                The URL of the web document to be queried for.
-                Cannot be used together with 'username'.
+        @param url: The URL of the web document to be queried for.
+            Cannot be used together with 'username'.
+        @type url: str
 
-            username
-                The username to be queried for.
-                Cannot be used together with 'url'.
+        @param username: The Delicious.com username to be queried for.
+            Cannot be used together with 'url'.
+        @type username: str
 
-            max_bookmarks (optional, default: 50)
-                Maximum number of bookmarks to retrieve. Set to 0 to disable
-                this limitation/the maximum and retrieve all available
-                bookmarks of the given url.
+        @param max_bookmarks: Optional, default: 50.
+            Maximum number of bookmarks to retrieve. Set to 0 to disable
+            this limitation/the maximum and retrieve all available
+            bookmarks of the given url.
 
-                Bookmarks are sorted so that newer bookmarks are first.
-                Setting max_bookmarks to 50 means that get_bookmarks() will retrieve
-                the 50 most recent bookmarks of the given url.
+            Bookmarks are sorted so that newer bookmarks are first.
+            Setting max_bookmarks to 50 means that get_bookmarks() will retrieve
+            the 50 most recent bookmarks of the given url.
 
-                In the case of getting bookmarks of a URL (url is set),
-                get_bookmarks() will take *considerably* longer to run
-                for pages with lots of bookmarks when setting max_bookmarks
-                to a high number or when you completely disable the limit.
-                Delicious returns only up to 50 bookmarks per result page,
-                so for example retrieving 250 bookmarks requires 5 HTTP
-                connections and parsing 5 HTML pages plus wait time between
-                queries (to comply with delicious' Terms of Use; see
-                also parameter 'sleep_seconds').
+            In the case of getting bookmarks of a URL (url is set),
+            get_bookmarks() will take *considerably* longer to run
+            for pages with lots of bookmarks when setting max_bookmarks
+            to a high number or when you completely disable the limit.
+            Delicious returns only up to 50 bookmarks per result page,
+            so for example retrieving 250 bookmarks requires 5 HTTP
+            connections and parsing 5 HTML pages plus wait time between
+            queries (to comply with delicious' Terms of Use; see
+            also parameter 'sleep_seconds').
 
-                In the case of getting bookmarks of a user (username is set),
-                the same restrictions as for a URL apply with the exception
-                that we can retrieve up to 100 bookmarks per HTTP query
-                (instead of only up to 50 per HTTP query for a URL).
+            In the case of getting bookmarks of a user (username is set),
+            the same restrictions as for a URL apply with the exception
+            that we can retrieve up to 100 bookmarks per HTTP query
+            (instead of only up to 50 per HTTP query for a URL).
+        @type max_bookmarks: int
 
-            sleep_seconds (optional, default: 1)
+        @param sleep_seconds: Optional, default: 1.
                 Wait the specified number of seconds between subsequent
                 queries in case that there are multiple pages of bookmarks
-                for the given url. Must be greater than or equal to 1 to
-                comply with delicious' Terms of Use.
+                for the given url. sleep_seconds must be >= 1 to comply with
+                Delicious.com's Terms of Use.
                 See also parameter 'max_bookmarks'.
+        @type sleep_seconds: int
+
+        @return: Returns the bookmarks of url or user, respectively.
+            For urls, it returns a list of (user, tags, comment, timestamp)
+            tuples.
+            For users, it returns a list of (url, tags, title, comment,
+            timestamp) tuples.
+
+            Bookmarks are sorted "descendingly" by creation time, i.e. newer
+            bookmarks come first.
 
         """
         # we must wait at least 1 second between subsequent queries to
@@ -626,7 +672,8 @@ class DeliciousAPI(object):
 
         path = None
         if url:
-            m = md5.new(url)
+            m = hashlib.md5()
+            m.update(url)
             hash = m.hexdigest()
 
             # path will change later on if there are multiple pages of boomarks
@@ -652,7 +699,7 @@ class DeliciousAPI(object):
                     bookmarks.extend(self._extract_bookmarks_from_user_history(data))
 
                 # stop scraping if we already have as many bookmarks as we want
-                if len(bookmarks) >= max_bookmarks:
+                if (len(bookmarks) >= max_bookmarks) and max_bookmarks != 0:
                     break
                 else:
                     # check if there are multiple pages of bookmarks for this
@@ -678,6 +725,17 @@ class DeliciousAPI(object):
 
 
     def _extract_bookmarks_from_url_history(self, data):
+        """
+        Extracts user bookmarks from a URL's history page on Delicious.com.
+
+        The Python library BeautifulSoup is used to parse the HTML page.
+
+        @param data: The HTML source of a URL history Web page on Delicious.com.
+        @type data: str
+
+        @return: list of user bookmarks of the corresponding URL
+
+        """
         bookmarks = []
         soup = BeautifulSoup(data)
 
@@ -738,6 +796,17 @@ class DeliciousAPI(object):
         return bookmarks
 
     def _extract_bookmarks_from_user_history(self, data):
+        """
+        Extracts a user's bookmarks from his user page on Delicious.com.
+
+        The Python library BeautifulSoup is used to parse the HTML page.
+
+        @param data: The HTML source of a user page on Delicious.com.
+        @type data: str
+
+        @return: list of bookmarks of the corresponding user
+
+        """
         bookmarks = []
         soup = BeautifulSoup(data)
 
@@ -803,23 +872,26 @@ class DeliciousAPI(object):
         This function can be used to backup all of a user's bookmarks if
         called with a username and password.
 
-        Parameters:
-            username:
-                The Delicious.com username.
+        @param username: The Delicious.com username.
+        @type username: str
 
-            password (optional, default: None)
-                The user's Delicious.com password. If password is set,
-                all communication with Delicious.com is SSL-encrypted.
+        @param password: Optional, default: None.
+            The user's Delicious.com password. If password is set,
+            all communication with Delicious.com is SSL-encrypted.
+        @type password: unicode/str
 
-            max_bookmarks (optional, default: 50)
-                See the documentation of get_bookmarks() for more
-                information as get_url() uses get_bookmarks() to
-                retrieve a url's bookmarking history.
+        @param max_bookmarks: Optional, default: 50.
+            See the documentation of get_bookmarks() for more
+            information as get_url() uses get_bookmarks() to
+            retrieve a url's bookmarking history.
+        @type max_bookmarks: int
 
-            sleep_seconds (optional, default: 1)
-                See the documentation of get_bookmarks() for more
-                information as get_url() uses get_bookmarks() to
-                retrieve a url's bookmarking history.
+        @param sleep_seconds: Optional, default: 1.
+            See the documentation of get_bookmarks() for more information as
+            get_url() uses get_bookmarks() to retrieve a url's bookmarking
+            history. sleep_seconds must be >= 1 to comply with Delicious.com's
+            Terms of Use.
+        @type sleep_seconds: int
 
         @return: DeliciousUser instance
 
@@ -903,7 +975,7 @@ class DeliciousAPI(object):
 
     def get_urls(self, tag=None, popular=True, max_urls=100, sleep_seconds=1):
         """
-        Returns the list of URLs (of web documents) for a given tag.
+        Returns the list of recent URLs (of web documents) tagged with a given tag.
 
         This is very similar to parsing Delicious' RSS/JSON feeds directly,
         but this function will return up to 2,000 links compared to a maximum
@@ -919,8 +991,8 @@ class DeliciousAPI(object):
 
         @param tag: Retrieve links which have been tagged with the given tag.
             If tag is not set (default), links will be retrieved from the
-            Delicious front page (aka "delicious hotlist").
-        @param type: unicode or str
+            Delicious.com front page (aka "delicious hotlist").
+        @type tag: unicode/str
 
         @param popular: If true (default), retrieve only popular links (i.e.
             /popular/<tag>). Otherwise, the most recent links tagged with
@@ -930,15 +1002,15 @@ class DeliciousAPI(object):
             of popular tags to contain only up to a maximum of 15 URLs.
             This also means that setting max_urls to values larger than 15
             will not change the results of get_urls().
-            So if you are interested in more URLs, set the popular parameter
+            So if you are interested in more URLs, set the "popular" parameter
             to false.
 
             Note that if you set popular to False, the returned list of URLs
-            might contain duplicate items. This is due to the way Delicious'
-            creates its /tag/<tag> web pages. So if you need a certain
+            might contain duplicate items. This is due to the way Delicious.com
+            creates its /tag/<tag> Web pages. So if you need a certain
             number of unique URLs, you have to take care of that in your
             own code.
-        @param type: bool
+        @type popular: bool
 
         @param max_urls: Retrieve at most max_urls links. The default is 100,
             which is the maximum number of links that can be retrieved by
@@ -946,14 +1018,17 @@ class DeliciousAPI(object):
             in practice is 2000 (currently). If it is set higher, Delicious
             will return the same links over and over again, giving lots of
             duplicate items.
-        @param type: int
+        @type max_urls: int
 
-        @param sleep_seconds: Wait the specified number of seconds between
-            subsequent queries in case that there are multiple pages of
-            bookmarks for the given url. Must be greater than or equal to 1
-            to comply with delicious' Terms of Use. The default value is 1.
+        @param sleep_seconds: Optional, default: 1.
+            Wait the specified number of seconds between subsequent queries in
+            case that there are multiple pages of bookmarks for the given url.
+            Must be greater than or equal to 1 to comply with Delicious.com's
+            Terms of Use.
             See also parameter 'max_urls'.
-        @param type: int
+        @type sleep_seconds: int
+
+        @return: The list of recent URLs (of web documents) tagged with a given tag.
 
         """
         assert sleep_seconds >= 1
@@ -1030,8 +1105,8 @@ class DeliciousAPI(object):
                             path = nexts[0]['href']
                             path += "&setcount=%d" % max_html_count
                             page_index += 1
-                            # wait one second between queries to be compliant with
-                            # delicious' Terms of Use
+                            # wait between queries to Delicious.com to be
+                            # compliant with its Terms of Use
                             time.sleep(sleep_seconds)
         if max_urls > 0:
             return urls[:max_urls]
@@ -1047,7 +1122,10 @@ class DeliciousAPI(object):
         DeliciousAPI uses the official JSON feed of the user. We could use
         RSS here, but the JSON feed has proven to be faster in practice.
 
-        Returns a dictionary mapping tags to their tag counts.
+        @param username: The Delicious.com username.
+        @type username: str
+
+        @return: Dictionary mapping tags to their tag counts.
 
         """
         tags = {}
@@ -1079,6 +1157,12 @@ class DeliciousAPI(object):
         characters).
 
         None is treated specially, and returns the empty string.
+
+        @param s: The string that needs to be escaped.
+        @type s: str
+
+        @return: The escaped string.
+
         """
         if s is None:
             return ''
@@ -1116,8 +1200,8 @@ class DeliciousUnknownError(DeliciousError):
 class DeliciousUnauthorizedError(DeliciousError):
     """Used to indicate that Delicious.com returned a 401 Unauthorized error.
 
-    Most of the time, the user credentials for acessing restricted (official)
-    Delicious.com API functions are incorrect.
+    Most of the time, the user credentials for accessing restricted functions
+    of the official Delicious.com API are incorrect.
 
     """
     pass
@@ -1140,8 +1224,7 @@ class DeliciousNotFoundError(DeliciousError):
 class Delicious500Error(DeliciousError):
     """Used to indicate that Delicious.com returned a 500 error.
 
-    Most of the time, retrying some seconds later fixes the problem
-    (because we only query existing pages with this API).
+    Most of the time, retrying some seconds later fixes the problem.
 
     """
     pass
